package ru.amse.vorobiev.lce.elements.impl;


import java.util.ArrayList;

import ru.amse.vorobiev.lce.circuit.ICircuit;
import ru.amse.vorobiev.lce.elements.*;

public abstract class AbstractGate implements IGate {
	private ICircuit myCircuit;
	private final ArrayList<IPlug> myInputs;
	private final ArrayList<IPlug> myOutputs;
	private int myInputCount;
	private int myOutputCount;
	
	protected AbstractGate(int inputCount, int outputCount, ICircuit circuit) {
		myCircuit = circuit;
		myInputs = new ArrayList<IPlug>();
		myOutputs = new ArrayList<IPlug>();
		myInputCount = inputCount;
		myOutputCount = outputCount;
		for (int i = 0; i < inputCount; i++) {
			myInputs.add(new Plug(this, PlugType.INPUT));
		}
		for (int i = 0; i < outputCount; i++) {
			myOutputs.add(new Plug(this, PlugType.OUTPUT));
		}
		if (myCircuit != null) {
			myCircuit.fireCircuitChanged();
		}
	}
	
	public ICircuit getCircuit() {
		return myCircuit;
	}
	
	public void setCircuit(ICircuit circuit) {
		myCircuit = circuit;
	}
	
	public IPlug getInput(int index) {
		return myInputs.get(index);
	}
	
	public IPlug getOutput(int index) {
		return myOutputs.get(index);
	}
	
	public int getInputCount() {
		return myInputCount;
	}
	
	public int getInputIndex(IPlug plug) {
		return myInputs.indexOf(plug);
	}
	
	public int getOutputIndex(IPlug plug) {
		return myOutputs.indexOf(plug);
	}
	
	public int getOutputCount() {
		return myOutputCount;
	}
	
	public IPlug addInput() {
		IPlug plug = new Plug(this, PlugType.INPUT);
		myInputs.add(plug);
		myInputCount++;
		myCircuit.fireCircuitChanged();
		return plug;
	}
	
	public IPlug addOutput() {
		IPlug plug = new Plug(this, PlugType.OUTPUT);
		myOutputs.add(plug);
		myOutputCount++;
		myCircuit.fireCircuitChanged();
		return plug;
	}
	
	public void deleteInput(int i) {
		myInputs.get(i).setWire(null);
		myInputs.remove(i);
		myInputCount--;
		myCircuit.fireCircuitChanged();
	}
	
	public void deleteOutput(int i) {
		myOutputs.get(i).setWire(null);
		myOutputs.remove(i);
		myOutputCount--;
		myCircuit.fireCircuitChanged();
	}
	
	public void fireGateChanged() {
		if (myCircuit != null) {
			myCircuit.fireCircuitChanged();
		}
	}
	
	public abstract GateType getType();
}
