package ru.amse.vorobiev.lce.io;

import java.awt.Point;
import java.io.File;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import ru.amse.vorobiev.lce.elements.IGate;
import ru.amse.vorobiev.lce.elements.IWire;
import ru.amse.vorobiev.lce.view.ICircuitView;
import ru.amse.vorobiev.lce.view.impl.CircuitView;
import ru.amse.vorobiev.lce.view.impl.GateView;
import ru.amse.vorobiev.lce.view.impl.WireView;

public class CircuitLoader {
	private ICircuitView myCircuitView;
	private SAXParser myParser;
	private CircuitModelLoader myModelLoader;
	
	public CircuitLoader() {
		myModelLoader = new CircuitModelLoader();
		SAXParserFactory factory = SAXParserFactory.newInstance();
		try {
			myParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void loadCircuit(File file) {
		myModelLoader.loadCircuitModel(file);
		myCircuitView = new CircuitView(myModelLoader.getCircuit());
		try {
			myParser.parse(file, new CircuitViewHandler());
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		for (IWire wire : myModelLoader.getWires()) {
			new WireView(wire, myCircuitView);
		}
	}
	
	public ICircuitView getCircuitView() {
		return myCircuitView;
	}
	
	private class CircuitViewHandler extends DefaultHandler {
		
		public void startElement(String namespaceURI, String localName,
		        	String qName, Attributes atts) throws SAXException {
			if (qName == "gateview") {
				Integer id = new Integer(atts.getValue("id"));
				Integer x = new Integer(atts.getValue("x"));
				Integer y = new Integer(atts.getValue("y"));
				IGate gate = myModelLoader.getMap().get(id);
				new GateView(new Point(x, y), gate, myCircuitView);
			}	
		}
	}
}
