package ru.amse.vorobiev.lce.io;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import ru.amse.vorobiev.lce.circuit.Circuit;
import ru.amse.vorobiev.lce.circuit.ICircuit;
import ru.amse.vorobiev.lce.elements.*;
import ru.amse.vorobiev.lce.elements.impl.*;

public class CircuitModelLoader {
	private ICircuit myCircuit;
	private SAXParser myParser;
	private Map<Integer, IGate> myIdToGateMap;
	private Set<IWire> myWires;
	
	public CircuitModelLoader() {
		myWires = new HashSet<IWire>();
		myCircuit = new Circuit();
		myIdToGateMap = new HashMap<Integer, IGate>();
		SAXParserFactory factory = SAXParserFactory.newInstance();
		try {
			myParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			throw new WrongFileFormatException();
		}
	}
	
	public void loadCircuitModel(File file) {
		try {
			myParser.parse(file, new CircuitModelHandler());
		} catch (SAXException e) {
			throw new WrongFileFormatException();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
	
	public ICircuit getCircuit() {
		return myCircuit;
	}
	
	public Map<Integer, IGate> getMap() {
		return myIdToGateMap;
	}
	
	public Set<IWire> getWires() {
		return myWires;
	}
	
	private class CircuitModelHandler extends DefaultHandler {
		
		public void startElement(String namespaceURI, String localName,
		        	String qName, Attributes atts) throws SAXException {
			if (qName == "gate") {
				Integer id = new Integer(atts.getValue("id"));
				String type = atts.getValue("type");
				String inputs = atts.getValue("inputs");
				String outputs = atts.getValue("outputs");
				IGate gate;
				if (type.equalsIgnoreCase("in")) {
					gate = new InGate(new Integer(outputs));
				} else if (type.equalsIgnoreCase("out")) {
					gate = new OutGate(new Integer(inputs));
				} else if (type.equalsIgnoreCase("and")) {
					gate = new AndGate(new Integer(inputs), new Integer(outputs));
				} else if (type.equalsIgnoreCase("or")) {
					gate = new OrGate(new Integer(inputs), new Integer(outputs));
				} else if (type.equalsIgnoreCase("xor")) {
					gate = new XorGate(new Integer(inputs), new Integer(outputs));
				} else if (type.equalsIgnoreCase("not")) {
					gate = new NotGate();
				} else {
					gate = null;
				}
				myCircuit.addGate(gate);
				myIdToGateMap.put(id, gate);
			} else if (qName.equalsIgnoreCase("wire")) {
				IGate from = myIdToGateMap.get(new Integer(atts.getValue("from")));
				Integer input = new Integer(atts.getValue("input"));
				IGate to = myIdToGateMap.get(new Integer(atts.getValue("to")));
				Integer output = new Integer(atts.getValue("output"));
				IPlug start;
				IPlug end;
				try {
					start = from.getOutput(output);
					end = to.getInput(input);
				} catch (IndexOutOfBoundsException e) {
					throw new WrongFileStructureException();
				}
				myWires.add(new Wire(start, end));
			}
		}
	}
}
