package ru.amse.vorobiev.lce.ui.tools.impl;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;

import ru.amse.vorobiev.lce.ui.JEditor;
import ru.amse.vorobiev.lce.ui.tools.ITool;
import ru.amse.vorobiev.lce.view.IGateView;
import ru.amse.vorobiev.lce.view.impl.CircuitView;

public class DeletingInputOutputTool implements ITool {
	private JEditor myEditor;
	private IGateView mySelectedGate;
	private int myStatus;
	private int mySelectedPlug;
	
	public DeletingInputOutputTool(JEditor editor) {
		myEditor = editor;
		mySelectedPlug = -1;
	}

	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		if (mySelectedGate != null) {
			g2.setColor(Color.GRAY);
			switch (myStatus) {
				case CircuitView.ABLE_TO_DELETE_INPUT_PLUG:
					Point input = mySelectedGate.getInputs().get(mySelectedPlug);
					g2.fillRect(input.x, input.y - 2, 
							JEditor.PLUG_DELETING_TOOL_WIDTH, JEditor.PLUG_DELETING_TOOL_HEIGHT);
					break;
				case CircuitView.ABLE_TO_DELETE_OUTPUT_PLUG:
					Point output = mySelectedGate.getOutputs().get(mySelectedPlug);
					g2.fillRect(output.x - JEditor.PLUG_DELETING_TOOL_WIDTH, output.y - 2, 
							JEditor.PLUG_DELETING_TOOL_WIDTH, JEditor.PLUG_DELETING_TOOL_HEIGHT);
					break;
			}
		}
	}

	public void mouseClicked(MouseEvent arg0) {
		switch (myStatus) {
			case CircuitView.ABLE_TO_DELETE_INPUT_PLUG:
				mySelectedGate.deleteInput(mySelectedPlug);
				break;
			case CircuitView.ABLE_TO_DELETE_OUTPUT_PLUG:
				mySelectedGate.deleteOutput(mySelectedPlug);
				break;
		}
		myStatus = CircuitView.UNABLE_TO_DELETE_PLUG;
		mySelectedPlug = -1;
		myEditor.repaint();
	}

	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseDragged(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseMoved(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		mySelectedGate = myEditor.getView().getGateForLocation(x, y);
		if (mySelectedGate != null) {
			myStatus = myEditor.getView().ableToDeletePlugInLocation(x, y, mySelectedGate);
			switch (myStatus) {
				case CircuitView.ABLE_TO_DELETE_INPUT_PLUG: {
					mySelectedPlug = myEditor.getView().getInputPlugToDelete(x, y, mySelectedGate);
					break;
				}
				case CircuitView.ABLE_TO_DELETE_OUTPUT_PLUG: {
					mySelectedPlug = myEditor.getView().getOutputPlugToDelete(x, y, mySelectedGate);
					break;
				}
				case CircuitView.UNABLE_TO_DELETE_PLUG: {
					mySelectedPlug = -1;
					break;
				}
			}
		} else {
			myStatus = CircuitView.UNABLE_TO_DELETE_PLUG;
		}
		myEditor.repaint();
	}

}
