package ru.amse.vorobiev.lce.ui.tools.impl;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.event.MouseEvent;

import ru.amse.vorobiev.lce.elements.IPlug;
import ru.amse.vorobiev.lce.elements.IWire;
import ru.amse.vorobiev.lce.elements.impl.Wire;
import ru.amse.vorobiev.lce.ui.JEditor;
import ru.amse.vorobiev.lce.ui.tools.ITool;
import ru.amse.vorobiev.lce.view.ICircuitView;
import ru.amse.vorobiev.lce.view.IPlugView;
import ru.amse.vorobiev.lce.view.impl.WireView;

public class WireSelectionTool implements ITool{
	private JEditor myEditor;
	private IPlugView myCoveredPlug;
	private IPlugView mySelectedPlug;
	private Point myMousePosition;
	
	public WireSelectionTool(JEditor editor) {
		myEditor = editor;
		myCoveredPlug = null;
		mySelectedPlug = null;
		myMousePosition = new Point(0,0);
	}

	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		
		if (myCoveredPlug != null) {
			g2.setColor(Color.GRAY);
			g2.drawRect(myCoveredPlug.getCenter().x - JEditor.PLUG_DIAMETER/2 - 1, myCoveredPlug.getCenter().y - JEditor.PLUG_DIAMETER/2 - 1, 
					JEditor.PLUG_DIAMETER + 2, JEditor.PLUG_DIAMETER + 2);
		}
		if (mySelectedPlug != null) {
			g2.setColor(Color.BLACK);
			Stroke thindashed = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, 
												BasicStroke.JOIN_BEVEL, 1.0f, 
												new float[] { 8.0f, 3.0f}, 0.0f);
			g2.setStroke(thindashed);
			g2.drawLine(mySelectedPlug.getCenter().x, mySelectedPlug.getCenter().y, myMousePosition.x, myMousePosition.y);
		}
	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		if (myEditor.getView() != null) {
			if (myCoveredPlug != null) {
				myMousePosition.x = e.getX();
				myMousePosition.y = e.getY();
				mySelectedPlug = myCoveredPlug;
				IPlug plug = mySelectedPlug.getPlug();
				if (mySelectedPlug.getPlug().getWire() != null) {
					ICircuitView circuitView = myEditor.getView();
					switch (plug.getType()) {
					case INPUT: {
						mySelectedPlug = (IPlugView) circuitView.getViewByElement(plug.getWire().getFrom());
						break;
					}
					case OUTPUT: {
						mySelectedPlug = (IPlugView) circuitView.getViewByElement(plug.getWire().getTo());
						break;
					}
					}
				}
			}
		}
	}

	public void mouseReleased(MouseEvent e) {
		if (mySelectedPlug != null) {
			IPlug coveredPlug = (myCoveredPlug != null) ? myCoveredPlug.getPlug() : null;
			IPlug selectedPlug = mySelectedPlug.getPlug();
			if (myCoveredPlug != null && 
				coveredPlug.getType() != selectedPlug.getType() &&
				coveredPlug.getWire() == null) {
				IWire wire = selectedPlug.getWire();
				if (wire != null) {
					switch (coveredPlug.getType()) {
						case INPUT: {
							wire.setTo(coveredPlug);
							break;
						}
						case OUTPUT: {
							wire.setFrom(coveredPlug);
							break;
						}
					}
				} else {
					switch (selectedPlug.getType()) {
						case INPUT: {
							wire = new Wire(coveredPlug, selectedPlug);
							break;
						}
						case OUTPUT: {
							wire = new Wire(selectedPlug, coveredPlug);
							break;
						}
					}
					new WireView(wire, myEditor.getView());
				}
			} else if (coveredPlug == null || (coveredPlug != null && selectedPlug.getWire() != coveredPlug.getWire())) {
				IWire wire = selectedPlug.getWire();
				if (wire != null) {
					wire.getTo().setWire(null);
					wire.setTo(null);
					wire.getFrom().setWire(null);
					wire.setFrom(null);
					myEditor.getView().removeViewByElement(wire);
				}
			}
			mySelectedPlug = null;
			myEditor.repaint();
		}
	}

	public void mouseDragged(MouseEvent e) {
		if (mySelectedPlug != null) {
			myMousePosition.x = e.getX();
			myMousePosition.y = e.getY();
			myEditor.repaint();
			this.mouseMoved(e);
		}
	}

	public void mouseMoved(MouseEvent e) {
		if (myEditor.getView() != null) {
			int x = e.getX();
			int y = e.getY();
			
			ICircuitView view = myEditor.getView();
			if (view != null) {
				IPlugView plugView = view.getPlugForLocation(x, y);
				if (plugView != null) {
					myCoveredPlug = plugView;
					myEditor.repaint();
				} else if (myCoveredPlug != null){
					myCoveredPlug = null;
					myEditor.repaint();
				}
			}
		}
	}
}
