package ru.amse.vorobiev.lce.validator;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import java.util.List;
import ru.amse.vorobiev.lce.circuit.ICircuit;
import ru.amse.vorobiev.lce.elements.*;
import ru.amse.vorobiev.lce.elements.impl.*;

public class Validator implements IValidator {
	private List<IError> myErrors = new ArrayList<IError>();
	
	private void lookForEmptyPlugs(ICircuit circuit) {
		for (IGate gate : circuit.getAllGates()) {
			if (gate.getType() != GateType.IN) {
				for (int i = 0; i < gate.getInputCount(); i++) {
					IPlug input = gate.getInput(i);
					if (input.getWire() == null) {
						myErrors.add(new PlugError(input, "some input plug does not have wire"));
					}
				}
			}	
			if (gate.getType() != GateType.OUT) {
				for (int i = 0; i < gate.getOutputCount(); i++) {
					IPlug output = gate.getOutput(i);
					if (output.getWire() == null) {
						myErrors.add(new PlugError(output, "some output plug does not have wire"));
					}
				}
			}
		}
	}
	
	public List<IError> validate(ICircuit circuit) {
		myErrors.clear();
		lookForEmptyPlugs(circuit);
		if (myErrors.isEmpty()) {
			lookForCycles(circuit);
		}
		return myErrors;
	}
	
	private void lookForCycles(ICircuit circuit) {
		Set<IGate> visited = new HashSet<IGate>();
		for (InGate inGate : circuit.getInGates()) {
			dfs(inGate, visited);
		}
	}
	
	private void dfs(IGate gate, Set<IGate> visited) {
		for (int i = 0; i <  gate.getOutputCount(); i++) {
			IWire wire = gate.getOutput(i).getWire();
			IGate nextGate = wire.getTo().getGate();
			if (!visited.contains(nextGate)) {
				visited.add(gate);
				dfs(nextGate, visited);
			} else {
				myErrors.add(new StructureError(wire, "there is a cycle in the circuit"));
			}
		}
		visited.clear();
	}
}
