package ru.amse.vorobiev.lce.view.impl;

import java.awt.Graphics;
import java.awt.Point;
import java.util.HashMap;
import java.util.Map;

import ru.amse.vorobiev.lce.circuit.ICircuit;
import ru.amse.vorobiev.lce.elements.IElement;
import ru.amse.vorobiev.lce.elements.IGate;
import ru.amse.vorobiev.lce.elements.IPlug;
import ru.amse.vorobiev.lce.elements.IWire;
import ru.amse.vorobiev.lce.elements.impl.GateType;
import ru.amse.vorobiev.lce.ui.JEditor;
import ru.amse.vorobiev.lce.view.*;


public class CircuitView implements ICircuitView{
	public static final int UNABLE_TO_ADD_PLUG = -1;
	public static final int ABLE_TO_ADD_INPUT_PLUG = 0;
	public static final int ABLE_TO_ADD_OUTPUT_PLUG = 1;
	public static final int UNABLE_TO_DELETE_PLUG = -1;
	public static final int ABLE_TO_DELETE_INPUT_PLUG = 0;
	public static final int ABLE_TO_DELETE_OUTPUT_PLUG = 1;
	
	private ICircuit myCircuit;
	private Map<IElement, IElementView> myElementToViewMap = new HashMap<IElement, IElementView>();

	public CircuitView(ICircuit circuit) {
		myCircuit = circuit;
	}
	
	public ICircuit getCircuit() {
		return myCircuit;
	}
	
	public IElementView addViewByElement(IElement element, IElementView elementView) {
		return myElementToViewMap.put(element, elementView);
	}
	
	public IElementView getViewByElement(IElement element) {
		return myElementToViewMap.get(element);
	}
	
	public IElementView removeViewByElement(IElement element) {
		if (element instanceof IGateView) {
			IGate gate = ((IGateView) element).getGate();
			for (int i = 0; i < gate.getInputCount(); i++) {
				IPlug plug = gate.getInput(i);
				IWire wire = plug.getWire();
				this.removeViewByElement(wire);
				this.removeViewByElement(plug);
			}
			for (int i = 0; i < gate.getOutputCount(); i++) {
				IPlug plug = gate.getOutput(i);
				IWire wire = plug.getWire();
				this.removeViewByElement(wire);
				this.removeViewByElement(plug);
			}
		}
		return myElementToViewMap.remove(element);
	}
	
	public void paint(Graphics g) {
		for (IGate gate : myCircuit.getAllGates()) {
			getViewByElement(gate).paint(g);
		}
	}

	public IGateView getGateForLocation(int x, int y) {
		for (IGate gate : myCircuit.getAllGates()) {
			IGateView gateView = (IGateView) getViewByElement(gate);
			if (x >= gateView.getUpperLeftCorner().x && 
			x <= gateView.getUpperLeftCorner().x + gateView.getWidth() && 
			y >= gateView.getUpperLeftCorner().y && 
			y <= gateView.getUpperLeftCorner().y + gateView.getHeight()) {
				return gateView;
			}
		}
		return null;
	}
		
	public IPlugView getPlugForLocation(int x, int y){
		for (IGate gate : myCircuit.getAllGates()) {
			for (int i = 0; i < gate.getInputCount(); i++) {
				IPlug plug = gate.getInput(i);
				IPlugView plugView = (IPlugView) getViewByElement(plug);
				if (x >= plugView.getCenter().x - JEditor.PLUG_DIAMETER/2&& 
						x <= plugView.getCenter().x + JEditor.PLUG_DIAMETER/2 && 
						y >= plugView.getCenter().y - JEditor.PLUG_DIAMETER/2 && 
						y <= plugView.getCenter().y + JEditor.PLUG_DIAMETER/2) {
							return plugView;
						}
			}
			for (int i = 0; i < gate.getOutputCount(); i++) {
				IPlug plug = gate.getOutput(i);
				IPlugView plugView = (IPlugView) getViewByElement(plug);
				if (x >= plugView.getCenter().x - JEditor.PLUG_DIAMETER/2 && 
						x <= plugView.getCenter().x + JEditor.PLUG_DIAMETER/2 && 
						y >= plugView.getCenter().y - JEditor.PLUG_DIAMETER/2 && 
						y <= plugView.getCenter().y + JEditor.PLUG_DIAMETER/2) {
							return plugView;
						}
			}
		}
		return null;
	}

	public int ableToAddPlugInLocation(int x, int y, IGateView gateView) {
		if (gateView == null) {
				return CircuitView.UNABLE_TO_ADD_PLUG;
		}
		int upX = gateView.getUpperLeftCorner().x;
		int upY = gateView.getUpperLeftCorner().y;
		if (x > upX && x < upX + JEditor.PLUG_CONTROL_LENGTH &&
				y > upY + gateView.getHeight() - JEditor.PLUG_CONTROL_LENGTH &&
				y < upY + gateView.getHeight()) {
			if (!(gateView.getGate().getType() == GateType.IN || gateView.getGate().getType() == GateType.NOT)) {
				return CircuitView.ABLE_TO_ADD_INPUT_PLUG;
			}
		}
		if (x > upX + gateView.getWidth() - JEditor.PLUG_CONTROL_LENGTH && 
				x < upX + gateView.getWidth() &&
				y > upY + gateView.getHeight() - JEditor.PLUG_CONTROL_LENGTH &&
				y < upY + gateView.getHeight()) {
			if (!(gateView.getGate().getType() == GateType.OUT)) {
				return CircuitView.ABLE_TO_ADD_OUTPUT_PLUG;
			}
		}
		return CircuitView.UNABLE_TO_ADD_PLUG;
	}
	

	public int ableToDeletePlugInLocation(int x, int y, IGateView gateView) {
		if (gateView == null) {
				return CircuitView.UNABLE_TO_DELETE_PLUG;
		}
		for (Point input : gateView.getInputs()) {
			if (x >= input.x && x <= input.x + JEditor.PLUG_DELETING_TOOL_WIDTH
				&& y >= input.y - JEditor.PLUG_DELETING_TOOL_HEIGHT / 2 
				&& y <= input.y + JEditor.PLUG_DELETING_TOOL_HEIGHT / 2
				&& gateView.getInputs().size() > 1) {
				return CircuitView.ABLE_TO_DELETE_INPUT_PLUG;
			}
		}
		for (Point input : gateView.getOutputs()) {
			if (x <= input.x && x >= input.x - JEditor.PLUG_DELETING_TOOL_WIDTH
				&& y >= input.y - JEditor.PLUG_DELETING_TOOL_HEIGHT / 2 
				&& y <= input.y + JEditor.PLUG_DELETING_TOOL_HEIGHT / 2
				&& gateView.getOutputs().size() > 1) {
				return CircuitView.ABLE_TO_DELETE_OUTPUT_PLUG;
			}
		}
		return CircuitView.UNABLE_TO_DELETE_PLUG;
	}

	public int getInputPlugToDelete(int x, int y, IGateView gateView) {
		if (gateView == null) {
				return -1;
		}
		int i = 0;
		for (Point input : gateView.getInputs()) {
			if (x >= input.x && x <= input.x + JEditor.PLUG_DELETING_TOOL_WIDTH
				&& y >= input.y - JEditor.PLUG_DELETING_TOOL_HEIGHT / 2 
				&& y <= input.y + JEditor.PLUG_DELETING_TOOL_HEIGHT / 2) {
				return i;
			}
			i++;
		}
		return -1;
	}

	public int getOutputPlugToDelete(int x, int y, IGateView gateView) {
		if (gateView == null) {
				return -1;
		}
		int i = 0;
		for (Point output : gateView.getOutputs()) {
			if (x <= output.x && x >= output.x - JEditor.PLUG_DELETING_TOOL_WIDTH
				&& y >= output.y - JEditor.PLUG_DELETING_TOOL_HEIGHT / 2 
				&& y <= output.y + JEditor.PLUG_DELETING_TOOL_HEIGHT / 2) {
				return i;
			}
			i++;
		}
		return -1;
	}
	
}
