package ru.amse.vorobiev.lce.view.impl;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import ru.amse.vorobiev.lce.elements.IGate;
import ru.amse.vorobiev.lce.elements.IPlug;
import ru.amse.vorobiev.lce.elements.impl.GateType;
import ru.amse.vorobiev.lce.ui.JEditor;
import ru.amse.vorobiev.lce.view.ICircuitView;

public class GateView extends AbstractGateView {

	public GateView(Point point, int width, int height, IGate gate, ICircuitView circuitView) {
		super(point, width, height, gate, circuitView);
	}
	
	public GateView(Point point, IGate gate, ICircuitView circuitView) {
		super(point, JEditor.GATE_INITIAL_WIDTH, JEditor.GATE_INITIAL_HEIGHT, gate, circuitView);
	}

	@Override
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
	
		g2.setColor(new Color(255, 255, 255));
		g2.fillRect(myUpperLeftCorner.x, myUpperLeftCorner.y, myWidth, myHeight);
		g2.setColor(new Color(0, 0, 0));
		g2.drawRect(myUpperLeftCorner.x, myUpperLeftCorner.y, myWidth, myHeight);
		g2.drawString(myName, myUpperLeftCorner.x + 10, myUpperLeftCorner.y + 12);
		
		if (myGate.getType() != GateType.IN && myGate.getType() != GateType.NOT) {
			g2.drawRect(myUpperLeftCorner.x, myUpperLeftCorner.y + myHeight - JEditor.PLUG_CONTROL_LENGTH, 
				JEditor.PLUG_CONTROL_LENGTH, JEditor.PLUG_CONTROL_LENGTH);
			g2.drawLine(myUpperLeftCorner.x + JEditor.PLUG_CONTROL_LENGTH/2, 
				myUpperLeftCorner.y + myHeight - JEditor.PLUG_CONTROL_LENGTH + 2, 
				myUpperLeftCorner.x + JEditor.PLUG_CONTROL_LENGTH/2, 
				myUpperLeftCorner.y + myHeight - 2);
			g2.drawLine(myUpperLeftCorner.x + 2, 
				myUpperLeftCorner.y + myHeight - JEditor.PLUG_CONTROL_LENGTH/2, 
				myUpperLeftCorner.x + JEditor.PLUG_CONTROL_LENGTH - 2, 
				myUpperLeftCorner.y + myHeight - JEditor.PLUG_CONTROL_LENGTH/2);
		}
		
		if (myGate.getType() != GateType.OUT) {
			g2.drawRect(myUpperLeftCorner.x + myWidth - JEditor.PLUG_CONTROL_LENGTH, 
				myUpperLeftCorner.y + myHeight - JEditor.PLUG_CONTROL_LENGTH, 
				JEditor.PLUG_CONTROL_LENGTH, JEditor.PLUG_CONTROL_LENGTH);
			g2.drawLine(myUpperLeftCorner.x + myWidth - JEditor.PLUG_CONTROL_LENGTH/2, 
				myUpperLeftCorner.y + myHeight - JEditor.PLUG_CONTROL_LENGTH + 2, 
				myUpperLeftCorner.x + myWidth - JEditor.PLUG_CONTROL_LENGTH/2, 
				myUpperLeftCorner.y + myHeight - 2);
			g2.drawLine(myUpperLeftCorner.x + myWidth - JEditor.PLUG_CONTROL_LENGTH + 2, 
				myUpperLeftCorner.y + myHeight - JEditor.PLUG_CONTROL_LENGTH/2, 
				myUpperLeftCorner.x + myWidth - 2, 
				myUpperLeftCorner.y + myHeight - JEditor.PLUG_CONTROL_LENGTH/2);
		}
		
		if (myGate.getType() != GateType.IN) {
			for (Point startPoint : myInputs) {
				g2.drawLine(startPoint.x, startPoint.y,	
						startPoint.x - JEditor.GATE_TO_PLUG_WIRE_LENGTH, startPoint.y);
				if (myInputs.size() > 1) {
					g2.drawLine(startPoint.x + 2, startPoint.y,	
							startPoint.x + 4, startPoint.y);
					g2.drawRect(startPoint.x, startPoint.y - 2, 
							JEditor.PLUG_DELETING_TOOL_WIDTH, JEditor.PLUG_DELETING_TOOL_HEIGHT);
				}
			}
			for (int i = 0; i < myGate.getInputCount(); i++) {
				IPlug input = myGate.getInput(i);
				myCircuitView.getViewByElement(input).paint(g);
				if (input.getWire() != null) {
					myCircuitView.getViewByElement(input.getWire()).paint(g);
				}
			}
		}
		
		if (myGate.getType() != GateType.OUT) {
			for (Point startPoint : myOutputs) {
				g2.setColor(Color.BLACK);
				g2.drawLine(startPoint.x, startPoint.y,	
						startPoint.x + JEditor.GATE_TO_PLUG_WIRE_LENGTH, startPoint.y);
				if (myOutputs.size() > 1) {
					g2.drawLine(startPoint.x - 2, startPoint.y,	
							startPoint.x - 4, startPoint.y);
					g2.drawRect(startPoint.x - JEditor.PLUG_DELETING_TOOL_WIDTH, startPoint.y - 2, 
							JEditor.PLUG_DELETING_TOOL_WIDTH, JEditor.PLUG_DELETING_TOOL_HEIGHT);
				}
			}
			for (int i = 0; i < myGate.getOutputCount(); i++) {
				IPlug output = myGate.getOutput(i);
				myCircuitView.getViewByElement(output).paint(g);

			}
		}
		
	}

	public void setSelectedAsError(boolean condition) {
	}
}
